﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public class Main : MonoBehaviour
{
	private static Main _instance;
	public static Main Inst { get { if (_instance == null) { _instance = FindObjectOfType<Main>(); _instance.Setup(); } return _instance; } }
	
	//public enum GameState { DuringDay, EndOfDay }
	public enum GameScreen { RequestScreen, StockScreen, Transmission }

	public struct StockedItem
	{
		public ItemData data;
		public int quantity;
	}


	public struct TransmissionData
	{
		public string[] monsterInstructions;
		public string[] stockInstructions;
	}

	private TransmissionData transmissionData;

	public struct DailyHeroStats
	{
		public int heroesStart;
		public int heroesServed;
		public int heroesSurvived;
		public int heroesDead;
		public int heroesRemaining;
		public int heroBonus;
		public int goldEarned;
	}

	private DailyHeroStats dailyHeroStats;

	//public GameState gameState;
	public bool duringDay;

	public Shop shop;
	public SaleScreen saleScreen;
	public RequestScreen requestScreen;
	public StockRetakeScreen stockRetakeScreen;
	public ActiveCustomer activeCustomer;
	public TransmissionScreen transmissionScreen;

	public CatalogueData catalogueData;
	public CustomerData customerData;
	public BestiaryData bestiary;

	public int money;

	public bool DEBUG_provide10Stock;

	private StockedItem[] stock;
	private Hero[] allHeroes;
	//private Queue<Hero> queuedHeroes;
	private int activeHeroIndex;

	private int[] transmissionEnemyIndexes;
	private int[] transmissionStockIndexes;

	private List<int> lowValueEnemyIndexes;
	private List<int> medValueEnemyIndexes;
	private List<int> hiValueEnemyIndexes;

	private void Start()
	{
		if(_instance == null)
		{
			_instance = this;
			Setup();
		}
	}

	void Setup()
	{
		#region Set up stock
		stock = new StockedItem[catalogueData.items.Length];
		for(int i = 0; i < stock.Length; i++)
		{
			StockedItem stockedItem;
			stockedItem.data = catalogueData.items[i];
			stockedItem.quantity = 0;
			stock[i] = stockedItem;
		}
		#endregion

		#region Set up heroes/customers
		allHeroes = new Hero[10];
		for(int i = 0; i < allHeroes.Length; i++)
		{
			Hero.HeroData newHeroData;
			bool female = Random.value > 0.5f;
			newHeroData.female = female;
			int nameIndex = Random.Range(0, female ? customerData.FemaleHeroes.Length : customerData.MaleHeroes.Length);
			newHeroData.name = female ? customerData.FemaleHeroes[nameIndex] : customerData.MaleHeroes[nameIndex];
			int spriteIndex = Random.Range(0, female ? customerData.FemaleSprites.Length : customerData.MaleSprites.Length);
			newHeroData.sprite = female ? customerData.FemaleSprites[spriteIndex] : customerData.MaleSprites[spriteIndex];
			allHeroes[i] = new Hero(newHeroData);
		}
		#endregion

		lowValueEnemyIndexes = new List<int>();
		medValueEnemyIndexes = new List<int>();
		hiValueEnemyIndexes = new List<int>();
		for(int i = 0; i < bestiary.enemies.Length; i++)
		{
			var enemy = bestiary.enemies[i];
			switch(enemy.enemyLevel)
			{
				case EnemyData.EnemyLevel.Low:
					lowValueEnemyIndexes.Add(i);
					break;
				case EnemyData.EnemyLevel.Medium:
					medValueEnemyIndexes.Add(i);
					break;
				case EnemyData.EnemyLevel.High:
					hiValueEnemyIndexes.Add(i);
					break;
			}
		}

		transmissionData.monsterInstructions = new string[3];
		transmissionData.stockInstructions = new string[3];
		transmissionEnemyIndexes = new int[3];
		transmissionStockIndexes = new int[3];

		if (DEBUG_provide10Stock)
		{
			for (int i = 0; i < catalogueData.items.Length; i++)
			{
				var item = catalogueData.items[i];
				for (int j = 0; j < 10; j++) AddItemToStock(item);
			}
		}

		SetUpTransmission();
		transmissionScreen.SetupTransmission(transmissionData, dailyHeroStats);

		duringDay = false;
	}

	public void OpenScreen(int screenIndex)
	{
		var screen = (GameScreen)screenIndex;
		saleScreen.HideScreen();
		requestScreen.HideScreen();
		stockRetakeScreen.HideScreen();
		transmissionScreen.HideScreen();

		switch(screen)
		{
			case GameScreen.RequestScreen:
				requestScreen.ShowScreen();
				break;
			case GameScreen.StockScreen:
				if (duringDay) saleScreen.ShowScreen();
				else stockRetakeScreen.ShowScreen();
				break;
			case GameScreen.Transmission:
				transmissionScreen.ShowScreen();
				break;
		}
	}

	private void Update()
	{
		shop.money.text = money.ToString();
		shop.timeOfDay.text = duringDay ? "During Day" : "End of Day";
		if(!duringDay && Input.GetKeyDown(KeyCode.Space))
		{
			StartNewDay();
		}
	}

	public StockedItem[] GetItemsInCatalogue()
	{
		return stock;
	}

	public EnemyData[] GetEnemiesInBestiary()
	{
		return bestiary.enemies;
	}

	public StockedItem[] GetItemsInStock()
	{
		return stock.Where(x => x.quantity > 0).ToArray();
	}

	public Hero GetActiveCustomer()
	{
		return allHeroes[activeHeroIndex];
	}

	public void AddItemToStock(ItemData item)
	{
		for (int i = 0; i < stock.Length; i++)
		{
			if (stock[i].data.Name == item.Name)
			{
				stock[i].quantity++;
				break;
			}
		}
	}

	public void RemoveItemFromStock(ItemData item)
	{
		for (int i = 0; i < stock.Length; i++)
		{
			if (stock[i].data.Name == item.Name)
			{
				stock[i].quantity--;
				break;
			}
		}
	}

	public void StartNewDay()
	{
		activeHeroIndex = 0;

		for(int i = 0; i < allHeroes.Length; i++)
		{
			allHeroes[i].SetNewTarget(bestiary);
		}

		ShowNextCustomer();

		duringDay = true;
	}

	public void ShowNextCustomer()
	{
		activeCustomer.Setup(allHeroes[activeHeroIndex]);
		requestScreen.SetupRequest(allHeroes[activeHeroIndex], customerData);
	}

	public void FinaliseCustomerTransaction(List<ItemData> shoppingList)
	{
		money += saleScreen.CostOfShoppingList;
		allHeroes[activeHeroIndex].SetItems(shoppingList);

		for (int i = 0; i < shoppingList.Count; i++)
		{
			RemoveItemFromStock(shoppingList[i]);
		}

		activeHeroIndex++;

		if(activeHeroIndex >= allHeroes.Length)
		{
			EndDay();
		}
		else
		{
			ShowNextCustomer();
		}
	}

	public void EndDay()
	{
		duringDay = false;
		saleScreen.HideScreen();
		requestScreen.HideScreen();
		stockRetakeScreen.HideScreen();

		// Do all hero vs target calculations here - did they live or die?
		// Also need to update the transmission
		foreach(var hero in allHeroes)
		{
			hero.FightEnemy();
		}

		SetUpTransmission();
		transmissionScreen.SetupTransmission(transmissionData, dailyHeroStats);
	}

	public void SetUpTransmission()
	{
		int lowValIndex = Random.Range(0, lowValueEnemyIndexes.Count);
		int medValIndex = Random.Range(0, medValueEnemyIndexes.Count);
		int hiValIndex = Random.Range(0, hiValueEnemyIndexes.Count);

		for(int i = 0; i < 3; i++)
		{
			transmissionEnemyIndexes[i] = -1;
			transmissionStockIndexes[i] = -1;
			transmissionData.monsterInstructions[i] = string.Empty;
			transmissionData.stockInstructions[i] = string.Empty;
		}

		transmissionEnemyIndexes[0] = lowValueEnemyIndexes[lowValIndex];
		transmissionEnemyIndexes[1] = medValueEnemyIndexes[medValIndex];
		transmissionEnemyIndexes[2] = Random.value > 0.5f ? hiValueEnemyIndexes[hiValIndex] : -1;
		
		ItemData hiValItem = null;
		if (transmissionEnemyIndexes[2] >= 0)
		{
			var hiValEnemy = bestiary.enemies[transmissionEnemyIndexes[2]];
			int weaknessIndex = Random.Range(0, hiValEnemy.itemsWeakAgainst.Length);
			hiValItem = hiValEnemy.itemsWeakAgainst[weaknessIndex];
			for (int i = 0; i < catalogueData.items.Length; i++)
			{
				if (hiValItem.Name == catalogueData.items[i].Name)
				{
					transmissionStockIndexes[0] = i;
					break;
				}
			}
		}

		var medValEnemy = bestiary.enemies[transmissionEnemyIndexes[1]];
		int medValWeaknessIndex = Random.Range(0, medValEnemy.itemsWeakAgainst.Length);
		var medValWeakItem = medValEnemy.itemsWeakAgainst[medValWeaknessIndex];
		if(hiValItem == null || medValWeakItem.Name != hiValItem.Name)
		{
			for (int i = 0; i < catalogueData.items.Length; i++)
			{
				if (medValWeakItem.Name == catalogueData.items[i].Name)
				{
					transmissionStockIndexes[1] = i;
					break;
				}
			}
		}

		int mInstructIndex = 0;
		if(transmissionEnemyIndexes[0] != -1)
		{
			transmissionData.monsterInstructions[mInstructIndex] =
				string.Format("- There are an increase of '{0}' in the area",
					bestiary.enemies[transmissionEnemyIndexes[0]].Name);
			mInstructIndex++;
		}

		if(transmissionEnemyIndexes[1] != -1)
		{
			transmissionData.monsterInstructions[mInstructIndex] =
				string.Format("- No sign of any '{0}' in the area",
					bestiary.enemies[transmissionEnemyIndexes[1]].Name);
			mInstructIndex++;
		}

		if(transmissionEnemyIndexes[2] != -1)
		{
			transmissionData.monsterInstructions[mInstructIndex] =
				string.Format("- There is potential for a '{0}' to appear",
					bestiary.enemies[transmissionEnemyIndexes[2]].Name);
		}

		int sInstructIndex = 0;

		if (transmissionStockIndexes[0] != -1)
		{
			transmissionData.stockInstructions[sInstructIndex] =
				string.Format("- Prices for '{0}' have soared",
					catalogueData.items[transmissionStockIndexes[0]].Name);
			sInstructIndex++;
		}

		if (transmissionStockIndexes[1] != -1)
		{
			transmissionData.stockInstructions[sInstructIndex] =
				string.Format("- Demand for '{0}' have lowered so prices plummet",
					catalogueData.items[transmissionStockIndexes[1]].Name);
			sInstructIndex++;
		}
	}
}
