﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class SaleScreen : BaseScreen
{
	public Button stockTab, bestiaryTab;
	public RectTransform stockScreen, bestiaryScreen;
	public ItemSaleScreen[] items;
	public Image[] listItems;
	public Text listMoney;
	public EnemySaleScreen[] enemies;
	//public ItemData[] itemData;
	public Button sellButton;
	public Button noSaleButton;

	private bool bestiaryActive = false;
	private int stockPageIndex;
	private const int stockPageLength = 5;
	private int bestiaryPageIndex;
	private const int bestiaryPageLength = 4;
	
	private List<ItemData> shoppingList;

	public int CostOfShoppingList
	{
		get
		{
			int num = 0;
			foreach(var item in shoppingList)
			{
				num += item.MinValue;
			}
			return num;
		}
	}

	private void Start()
	{
		stockPageIndex = 0;

		UpdatePage();

		for (int i = 0; i < listItems.Length; i++)
		{
			listItems[i].gameObject.SetActive(false);
		}

		shoppingList = new List<ItemData>(3);

		UpdateShoppingList();
	}

	public override void ShowScreen()
	{
		base.ShowScreen();
		bestiaryActive = false;
		stockPageIndex = 0;
		bestiaryPageIndex = 0;
		UpdatePage();
	}

	public void ChangeTab(int tab)
	{
		bestiaryActive = tab == 1;
		UpdatePage();
	}

	public void IncrementPage()
	{
		if(!bestiaryActive)
		{
			var itemData = Main.Inst.GetItemsInStock();
			int maxPages = itemData.Length / stockPageLength + ((itemData.Length % stockPageLength > 0) ? 1 : 0);
			if (stockPageIndex + 1 == maxPages) return;
			stockPageIndex++;
		}
		else
		{
			var enemyData = Main.Inst.GetEnemiesInBestiary();
			int maxPages = enemyData.Length / bestiaryPageLength + ((enemyData.Length % bestiaryPageLength > 0) ? 1 : 0);
			if (bestiaryPageIndex + 1 == maxPages) return;
			bestiaryPageIndex++;
		}

		UpdatePage();
	}

	public void DecrementPage()
	{
		if(!bestiaryActive)
		{
			if (stockPageIndex == 0) return;
			stockPageIndex--;
		}
		else
		{
			if (bestiaryPageIndex == 0) return;
			bestiaryPageIndex--;
		}
		UpdatePage();
	}

	private void UpdatePage()
	{
		stockScreen.gameObject.SetActive(!bestiaryActive);
		bestiaryScreen.gameObject.SetActive(bestiaryActive);

		if(!bestiaryActive)
		{
			var itemData = Main.Inst.GetItemsInStock();
			int startIndex = stockPageIndex * stockPageLength;
			int endIndex = Mathf.Min(itemData.Length, (stockPageIndex + 1) * stockPageLength);
			int adjustedPageLength = endIndex - startIndex;

			for (int i = 0; i < adjustedPageLength; i++)
			{
				items[i].SetItemData(itemData[startIndex + i]);
			}

			int numToBlank = stockPageLength - adjustedPageLength;
			for (int i = stockPageLength - numToBlank; i < stockPageLength; i++)
			{
				items[i].Hide();
			}
		}
		else
		{
			var enemyData = Main.Inst.GetEnemiesInBestiary();
			int startIndex = bestiaryPageIndex * bestiaryPageLength;
			int endIndex = Mathf.Min(enemyData.Length, (bestiaryPageIndex + 1) * bestiaryPageLength);
			int adjustedPageLength = endIndex - startIndex;

			for (int i = 0; i < adjustedPageLength; i++)
			{
				enemies[i].SetEnemyData(enemyData[startIndex + i]);
			}

			int numToBlank = bestiaryPageLength - adjustedPageLength;
			for (int i = bestiaryPageLength - numToBlank; i < bestiaryPageLength; i++)
			{
				enemies[i].Hide();
			}
		}
		
	}

	public void AddItemToList(int itemIndex)
	{
		int numInList = shoppingList.Count;
		if (numInList == listItems.Length) return;

		var data = items[itemIndex].GetItemData();
		var customer = Main.Inst.GetActiveCustomer();
		if (CostOfShoppingList + data.MinValue > customer.GetCurrentMoney()) return;

		if (shoppingList.Exists(x => x.Name == data.Name)) return;

		shoppingList.Add(data);
		UpdateShoppingList();
	}

	public void RemoveItemFromList(int listIndex)
	{
		shoppingList.RemoveAt(listIndex);
		UpdateShoppingList();
	}

	public void UpdateShoppingList()
	{
		sellButton.gameObject.SetActive(shoppingList.Count > 0);
		noSaleButton.gameObject.SetActive(shoppingList.Count == 0);

		for(int i = 0; i < listItems.Length; i++)
		{
			if(i < shoppingList.Count)
			{
				listItems[i].gameObject.SetActive(true);
				listItems[i].sprite = shoppingList[i].Image;
				listItems[i].color = shoppingList[i].color;
			}
			else
			{
				listItems[i].sprite = null;
				listItems[i].gameObject.SetActive(false);
			}
		}
		
		listMoney.text = CostOfShoppingList.ToString();
	}

	public void SellShoppingList()
	{
		Main.Inst.FinaliseCustomerTransaction(shoppingList);
		shoppingList.Clear();
		UpdateShoppingList();
		UpdatePage();
	}

	public void NoSale()
	{
		shoppingList.Clear();
		Main.Inst.FinaliseCustomerTransaction(shoppingList);
		UpdateShoppingList();
		UpdatePage();
	}
}
